import sys
import logging
from pprint import pprint
import json
from pathlib import Path
import fileinput

import xmltodict
import argparse


def extract_prefixes (d):
    return { k.split(':')[1]: d[k] for k in d.keys() if k.startswith("@xmlns:") }

def removePrefixes (o, prefixes):
    if not isinstance(o, dict): return    
    replacements = []
    for k,v in o.items():
        if isinstance(v, dict):
            removePrefixes(v, prefixes)
        if isinstance(v, list):
            for i in v:
                removePrefixes(i, prefixes)
        if ":" in k:
            prefix = k.split(':')[0]
            if (prefix) in prefixes:
                new_key = k.split(':')[1]
                replacements.append( (k, new_key) )
    for r in replacements:
        o[r[1]] = o.pop(r[0])

object_namespaces = {
    'AuthorisationObject' : 'http://uri.etsi.org/03120/common/2020/09/Authorisation',
    'DeliveryObject' : 'http://uri.etsi.org/03120/common/2019/10/Delivery',
    'DocumentObject' : 'http://uri.etsi.org/03120/common/2020/09/Document',
    'NotificationObject' : 'http://uri.etsi.org/03120/common/2016/02/Notification',
    'LITaskObject' : 'http://uri.etsi.org/03120/common/2020/09/Task',
    'LPTaskObject' : 'http://uri.etsi.org/03120/common/2020/09/Task',
    'LDTaskObject' : 'http://uri.etsi.org/03120/common/2020/09/Task',
    'TrafficPolicyObject' : 'http://uri.etsi.org/03120/common/2022/07/TrafficPolicy',
    'TrafficRuleObject' : 'http://uri.etsi.org/03120/common/2022/07/TrafficPolicy'
}

coerce_to_list = [
    'auth:AuthorisationApprovalDetails',
    'auth:AuthorisationFlag',
    'auth:CSPID',
    'common:ApproverContactDetails',
    'ActionRequest',
    'ActionResponse',
    'ListResponseRecord',
    'AssociatedObject',
    'doc:DocumentSignature',
    'doc:DocumentProperty',
    'notification:AssociatedObjectStatus',
    'task:ApprovalDetails',
    'task:TargetIdentifierValue',
    'task:DeliveryDestination',
    'task:TaskFlag',
    'task:AlternativePreservationReference',
    'task:ApprovalDetails',
    'task:ObservedTimes',
    'task:RequestValue',
    'task:RequestSubtype',
    'task:LDDeliveryDestination',
    'task:LDTaskFlag',
    'task:TrafficPolicyReference',
    'tp:TrafficRuleReference',
    'tp:Criteria'
]

coerce_to_int = [
    'ActionIdentifier',
    'delivery:SequenceNumber',
    'task:Order'
]

coerce_to_bool = [
    'delivery:LastSequence'
]

def postprocessor (path, key, value):
    if key == "@xsi:type":
        object_name = value.split(":")[-1]
        if object_name in object_namespaces.keys():
            value = "{" + object_namespaces[object_name] + "}" + object_name
        return key, value
    if key in coerce_to_int:
        return key, int(value)
    if key in coerce_to_bool:
        return key, bool(value)
    return key, value

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('-v', '--verbose', action='count', help='Verbose logging (can be specified multiple times)')
    parser.add_argument('-i', '--input', type=argparse.FileType('r'), default=sys.stdin, help="Path to input file (if absent, stdin is used)")
    args = parser.parse_args()

    match args.verbose:
        case v if v and v >= 2:
            logging.basicConfig(level=logging.DEBUG)
        case 1:
            logging.basicConfig(level=logging.INFO)
        case _:
            logging.basicConfig(level=logging.WARNING)

    logging.debug(f"Arguments: {args}")

    s = args.input.read()
    args.input.close()

    logging.debug(s)

    d = xmltodict.parse(s,
                        force_list=tuple(coerce_to_list),
                        postprocessor=postprocessor
                        )['HI1Message']

    print(json.dumps(d, indent=2))
